==> jei-1.19.4-fabric-13.1.0.8-sources.jar: mezz/jei/gui/util/CommandUtil.java <==
package mezz.jei.gui.util;

import mezz.jei.common.network.IConnectionToServer;
import mezz.jei.common.network.packets.PacketGiveItemStack;
import mezz.jei.common.network.packets.PacketSetHotbarItemStack;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.ServerCommandUtil;
import net.minecraft.class_1799;
import net.minecraft.class_310;
import net.minecraft.class_481;
import net.minecraft.class_636;
import net.minecraft.class_746;
import mezz.jei.common.config.GiveMode;
import mezz.jei.common.config.IClientConfig;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public final class CommandUtil {
	private static final Logger LOGGER = LogManager.getLogger();
	private final IClientConfig clientConfig;
	private final IConnectionToServer serverConnection;

	public CommandUtil(IClientConfig clientConfig, IConnectionToServer serverConnection) {
		this.clientConfig = clientConfig;
		this.serverConnection = serverConnection;
	}

	/**
	 * /give &lt;player&gt; &lt;item&gt; [amount]
	 * <p>
	 * {@link class_481} has special client-side handling for itemStacks, just give the item on the client
	 */
	public void giveStack(class_1799 itemStack, GiveAmount giveAmount) {
		final GiveMode giveMode = clientConfig.getGiveMode();
		class_310 minecraft = class_310.method_1551();
		class_746 player = minecraft.field_1724;
		if (player == null) {
			LOGGER.error("Can't give stack, there is no player");
			return;
		}
		final int amount = giveAmount.getAmountForStack(itemStack);
		if (minecraft.field_1755 instanceof class_481 && giveMode == GiveMode.MOUSE_PICKUP) {
			class_1799 sendStack = copyWithSize(itemStack, amount);
			ServerCommandUtil.mousePickupItemStack(player, sendStack);
		} else if (serverConnection.isJeiOnServer()) {
			class_1799 sendStack = copyWithSize(itemStack, amount);
			PacketGiveItemStack packet = new PacketGiveItemStack(sendStack, giveMode);
			serverConnection.sendPacketToServer(packet);
		} else {
			giveStackVanilla(itemStack, amount);
		}
	}

	public void setHotbarStack(class_1799 itemStack, int hotbarSlot) {
		if (serverConnection.isJeiOnServer()) {
			class_1799 sendStack = copyWithSize(itemStack, itemStack.method_7914());
			PacketSetHotbarItemStack packet = new PacketSetHotbarItemStack(sendStack, hotbarSlot);
			serverConnection.sendPacketToServer(packet);
		}
	}

	/**
	 * Fallback for when JEI is not on the server.
	 * Uses the Creative Inventory Action Packet when in creative, which doesn't require the player to be op.
	 */
	private static void giveStackVanilla(class_1799 itemStack, int amount) {
		if (itemStack.method_7960()) {
			String stackInfo = ErrorUtil.getItemStackInfo(itemStack);
			LOGGER.error("Empty itemStack: {}", stackInfo, new IllegalArgumentException());
			return;
		}

		class_746 sender = class_310.method_1551().field_1724;
		if (sender != null) {
			if (sender.method_7337()) {
				sendCreativeInventoryActions(sender, itemStack, amount);
			}
		}
	}

	private static void sendCreativeInventoryActions(class_746 sender, class_1799 stack, int amount) {
		int i = 0; // starting in the inventory, not armour or crafting slots
		while (i < sender.method_31548().field_7547.size() && amount > 0) {
			class_1799 currentStack = sender.method_31548().field_7547.get(i);
			if (currentStack.method_7960()) {
				class_1799 sendAllRemaining = copyWithSize(stack, amount);
				sendSlotPacket(sendAllRemaining, i);
				amount = 0;
			} else if (currentStack.method_7962(stack) && currentStack.method_7914() > currentStack.method_7947()) {
				int canAdd = Math.min(currentStack.method_7914() - currentStack.method_7947(), amount);
				class_1799 fillRemainingSpace = copyWithSize(stack, canAdd + currentStack.method_7947());
				sendSlotPacket(fillRemainingSpace, i);
				amount -= canAdd;
			}
			i++;
		}
		if (amount > 0) {
			class_1799 toDrop = copyWithSize(stack, amount);
			sendSlotPacket(toDrop, -1);
		}
	}

	private static void sendSlotPacket(class_1799 stack, int mainInventorySlot) {
		if (mainInventorySlot < 9 && mainInventorySlot != -1) {
			// slot ID for the message is different from the slot id used in the mainInventory
			mainInventorySlot += 36;
		}
		class_310 minecraft = class_310.method_1551();
		class_636 playerController = minecraft.field_1761;
		if (playerController != null) {
			playerController.method_2909(stack, mainInventorySlot);
		} else {
			LOGGER.error("Cannot send slot packet, minecraft.playerController is null");
		}
	}

	private static class_1799 copyWithSize(class_1799 itemStack, int size) {
		if (size == 0) {
			return class_1799.field_8037;
		}
		class_1799 copy = itemStack.method_7972();
		copy.method_7939(size);
		return copy;
	}
}

==> jei-1.20-fabric-14.0.0.9-sources.jar: mezz/jei/gui/util/CommandUtil.java <==
package mezz.jei.gui.util;

import mezz.jei.common.network.IConnectionToServer;
import mezz.jei.common.network.packets.PacketGiveItemStack;
import mezz.jei.common.network.packets.PacketSetHotbarItemStack;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.ServerCommandUtil;
import net.minecraft.class_1799;
import net.minecraft.class_310;
import net.minecraft.class_481;
import net.minecraft.class_636;
import net.minecraft.class_746;
import mezz.jei.common.config.GiveMode;
import mezz.jei.common.config.IClientConfig;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public final class CommandUtil {
	private static final Logger LOGGER = LogManager.getLogger();
	private final IClientConfig clientConfig;
	private final IConnectionToServer serverConnection;

	public CommandUtil(IClientConfig clientConfig, IConnectionToServer serverConnection) {
		this.clientConfig = clientConfig;
		this.serverConnection = serverConnection;
	}

	/**
	 * /give &lt;player&gt; &lt;item&gt; [amount]
	 * <p>
	 * {@link class_481} has special client-side handling for itemStacks, just give the item on the client
	 */
	public void giveStack(class_1799 itemStack, GiveAmount giveAmount) {
		final GiveMode giveMode = clientConfig.getGiveMode();
		class_310 minecraft = class_310.method_1551();
		class_746 player = minecraft.field_1724;
		if (player == null) {
			LOGGER.error("Can't give stack, there is no player");
			return;
		}
		final int amount = giveAmount.getAmountForStack(itemStack);
		if (minecraft.field_1755 instanceof class_481 && giveMode == GiveMode.MOUSE_PICKUP) {
			class_1799 sendStack = copyWithSize(itemStack, amount);
			ServerCommandUtil.mousePickupItemStack(player, sendStack);
		} else if (serverConnection.isJeiOnServer()) {
			class_1799 sendStack = copyWithSize(itemStack, amount);
			PacketGiveItemStack packet = new PacketGiveItemStack(sendStack, giveMode);
			serverConnection.sendPacketToServer(packet);
		} else {
			giveStackVanilla(itemStack, amount);
		}
	}

	public void setHotbarStack(class_1799 itemStack, int hotbarSlot) {
		if (serverConnection.isJeiOnServer()) {
			class_1799 sendStack = copyWithSize(itemStack, itemStack.method_7914());
			PacketSetHotbarItemStack packet = new PacketSetHotbarItemStack(sendStack, hotbarSlot);
			serverConnection.sendPacketToServer(packet);
		}
	}

	/**
	 * Fallback for when JEI is not on the server.
	 * Uses the Creative Inventory Action Packet when in creative, which doesn't require the player to be op.
	 */
	private static void giveStackVanilla(class_1799 itemStack, int amount) {
		if (itemStack.method_7960()) {
			String stackInfo = ErrorUtil.getItemStackInfo(itemStack);
			LOGGER.error("Empty itemStack: {}", stackInfo, new IllegalArgumentException());
			return;
		}

		class_746 sender = class_310.method_1551().field_1724;
		if (sender != null) {
			if (sender.method_7337()) {
				sendCreativeInventoryActions(sender, itemStack, amount);
			}
		}
	}

	private static void sendCreativeInventoryActions(class_746 sender, class_1799 stack, int amount) {
		int i = 0; // starting in the inventory, not armour or crafting slots
		while (i < sender.method_31548().field_7547.size() && amount > 0) {
			class_1799 currentStack = sender.method_31548().field_7547.get(i);
			if (currentStack.method_7960()) {
				class_1799 sendAllRemaining = copyWithSize(stack, amount);
				sendSlotPacket(sendAllRemaining, i);
				amount = 0;
			} else if (class_1799.method_7984(currentStack, stack) && currentStack.method_7914() > currentStack.method_7947()) {
				int canAdd = Math.min(currentStack.method_7914() - currentStack.method_7947(), amount);
				class_1799 fillRemainingSpace = copyWithSize(stack, canAdd + currentStack.method_7947());
				sendSlotPacket(fillRemainingSpace, i);
				amount -= canAdd;
			}
			i++;
		}
		if (amount > 0) {
			class_1799 toDrop = copyWithSize(stack, amount);
			sendSlotPacket(toDrop, -1);
		}
	}

	private static void sendSlotPacket(class_1799 stack, int mainInventorySlot) {
		if (mainInventorySlot < 9 && mainInventorySlot != -1) {
			// slot ID for the message is different from the slot id used in the mainInventory
			mainInventorySlot += 36;
		}
		class_310 minecraft = class_310.method_1551();
		class_636 playerController = minecraft.field_1761;
		if (playerController != null) {
			playerController.method_2909(stack, mainInventorySlot);
		} else {
			LOGGER.error("Cannot send slot packet, minecraft.playerController is null");
		}
	}

	private static class_1799 copyWithSize(class_1799 itemStack, int size) {
		if (size == 0) {
			return class_1799.field_8037;
		}
		class_1799 copy = itemStack.method_7972();
		copy.method_7939(size);
		return copy;
	}
}
